import { ref, reactive, watch, h } from 'vue';
import { useForm as useForm$1 } from '@inertiajs/inertia-vue3';

const useIndex = function useIndex({ route, syncUrl = false, defaultPerPage = 10, filters = {}, transformFilters }) {
    const search = ref("");
    let index = reactive({
        processing: false,
        perPage: defaultPerPage,
        items: [],
        hasNextPage: false,
        hasPrevPage: false,
        currentPage: 1,
        search: search,
        fromItem: 0,
        toItem: 0,
        totalItems: 0,
        __route: route,
        __hooks: {
            beforeUpdate: [],
            afterUpdate: [],
        },
        filters: reactive(filters),
        loadItems() {
            this.processing = true;
            for (let i in this.__hooks.beforeUpdate) {
                this.__hooks.beforeUpdate[i](this);
            }
            fetch(this.__getUrl())
                .then(response => response.json())
                .then((data) => this.__update(data))
                .catch(() => this.processing = false);
        },
        reloadOnChange(value) {
            watch(value, () => this.reload());
        },
        reload() {
            return this.loadItems();
        },
        addFilter(filter) {
            this.filters.push(filter);
            this.reload();
        },
        removeFilter(filter) {
            //
            this.reload();
        },
        getLastPage() {
            return Math.ceil(this.totalItems / this.perPage);
        },
        setPage(newPage) {
            if (this.newPage < 1 || this.newPage > this.getLastPage()) {
                return;
            }
            this.currentPage = newPage;
            this.reload();
        },
        nextPage() {
            this.setPage(this.currentPage + 1);
        },
        prevPage() {
            this.setPage(this.currentPage - 1);
        },
        lastPage() {
            this.setPage(this.getLastPage());
        },
        updateSearch(e) {
            console.log({ e });
            if (e instanceof Event) {
                this.search = e.target.value;
            }
            else {
                this.search = e;
            }
            this.setPage(1);
        },
        beforeUpdate(cb) {
            this.__hooks.beforeUpdate.push(cb);
        },
        onUpdate(cb) {
            this.__hooks.afterUpdate.push(cb);
        },
        __update(data) {
            this.items = data.data;
            this.totalItems = data.meta.total;
            this.fromItem = data.meta.from;
            this.toItem = data.meta.to;
            this.perPage = data.meta.per_page;
            this.hasNextPage = this.currentPage < this.getLastPage();
            this.hasPrevPage = this.currentPage > 1;
            this.processing = false;
            this.__syncUrl();
            for (let i in this.__hooks.afterUpdate) {
                this.__hooks.afterUpdate[i](this);
            }
        },
        __syncUrl() {
            if (!syncUrl) {
                return;
            }
            let prefix = '';
            if (typeof syncUrl === 'string') {
                prefix = `${syncUrl}`;
            }
            let params = {};
            let p = this.__getParams();
            for (let key in p) {
                params[prefix + key] = p[key];
            }
            console.log(params);
        },
        __getUrl() {
            const paramsString = new URLSearchParams(this.__getParams()).toString();
            return `${this.__route}?${paramsString}`;
        },
        __getParams() {
            let params = {
                page: this.currentPage,
                search: this.search,
            };
            let filters = JSON.parse(JSON.stringify(this.filters));
            if (transformFilters) {
                filters = transformFilters(filters);
            }
            for (let key in filters) {
                if (!filters[key]) {
                    continue;
                }
                params[`filter.${key}`] = filters[key];
            }
            for (let key in params) {
                params[key] = encodeURIComponent(params[key]);
            }
            return params;
        }
    });
    index.reloadOnChange(search);
    return index;
};

const useForm = function (route, model, { method = 'put', onCancelToken = () => { }, onBefore = () => { }, onStart = () => { }, onProgress = () => { }, onFinish = () => { }, onCancel = () => { }, onSuccess = () => { }, onError = () => { }, } = {}) {
    const inertiaForm = useForm$1(model);
    let form = reactive({
        ...inertiaForm,
        submit(e) {
            if (e instanceof Event) {
                e.preventDefault();
            }
            this.__submit(method, route, {
                headers: { Accept: 'application/json' },
                onCancelToken,
                onBefore,
                onStart,
                onProgress,
                onFinish,
                onCancel,
                onSuccess,
                onError
            });
        },
        get: undefined,
        post: undefined,
        put: undefined,
        patch: undefined,
        delete: undefined,
        __submit: inertiaForm.submit
    });
    return form;
};

const debounce = require('lodash.debounce');
const Input = function ({ modelModifiers = {} }, { attrs = {}, emit }) {
    let onInput = ({ target }) => {
        emit('update:modelValue', target.value);
    };
    if (modelModifiers.debounce) {
        onInput = debounce(onInput, 200);
    }
    return h(`input`, {
        ...attrs,
        value: attrs.modelValue,
        onInput,
    });
};

export { Input, useForm, useIndex };
